package org.joget.pbuilder;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.joget.apps.app.dao.BuilderDefinitionDao;
import org.joget.apps.app.model.AppDefinition;
import org.joget.apps.app.model.BuilderDefinition;
import org.joget.apps.app.model.CustomBuilder;
import org.joget.apps.app.model.CustomBuilderAbstract;
import org.joget.apps.app.service.AppPluginUtil;
import org.joget.apps.app.service.AppUtil;
import org.joget.apps.app.service.CustomBuilderUtil;
import org.joget.apps.userview.model.Permission;
import org.joget.apps.userview.service.UserviewUtil;
import org.joget.commons.util.LogUtil;
import org.joget.commons.util.SecurityUtil;
import org.joget.commons.util.StringUtil;
import org.joget.directory.model.User;
import org.joget.pbuilder.api.PageWidget;
import org.joget.plugin.base.Plugin;
import org.joget.plugin.base.PluginManager;
import org.joget.plugin.base.PluginWebSupport;
import org.joget.plugin.property.model.PropertyEditable;
import org.joget.plugin.property.service.PropertyUtil;
import org.joget.workflow.model.service.WorkflowUserManager;
import org.joget.workflow.util.WorkflowUtil;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

public class PageBuilder extends CustomBuilderAbstract implements CustomBuilder, PluginWebSupport {

    public final static String MESSAGE_PATH = "messages/PageBuilder";
    
    @Override
    public String getName() {
        return "pageBuilder";
    }
    
    @Override
    public String getObjectName() {
        return "page";
    }
    
    @Override
    public String getVersion() {
        return "7.0.0";
    }

    @Override
    public String getDescription() {
        return AppPluginUtil.getMessage("pageBuilder.description", getClassName(), getResourceBundlePath());
    }

    @Override
    public String getIcon() {
        return "fas fa-columns";
    }

    @Override
    public String getColor() {
        return "#43a047";
    }
    
    @Override
    public String getClassName() {
        return getClass().getName();
    }

    @Override
    public String getLabel() {
        return AppPluginUtil.getMessage("pageBuilder.label", getClassName(), getResourceBundlePath());
    }
    
    @Override
    public String getObjectLabel() {
        return AppPluginUtil.getMessage("pageBuilder.page", getClassName(), getResourceBundlePath());
    }

    @Override
    public String getIdPrefix() {
        return "page_";
    }
    
    @Override
    public String getPropertyOptions() {
        return AppUtil.readPluginResource(getClassName(), "/setting.json", null, true, getResourceBundlePath());
    }

    @Override
    public String getBuilderConfig() {
        return AppUtil.readPluginResource(getClassName(), "/page_builder.json", null, true, getResourceBundlePath());
    }

    @Override
    public String getResourceBundlePath() {
        return MESSAGE_PATH;
    }

    @Override
    public String getBuilderJS(String contextPath, String buildNumber) {
        return "<script type=\"text/javascript\" src=\""+contextPath+"/plugin/org.joget.pbuilder.PageBuilder/pagebuilder.js?build="+getVersion()+buildNumber+"\"></script>";
    }

    @Override
    public String getBuilderCSS(String contextPath, String buildNumber) {
        String css = "<link href=\""+contextPath+"/plugin/org.joget.pbuilder.PageBuilder/pagebuilder.css?build="+getVersion()+buildNumber+"\" rel=\"stylesheet\" type=\"text/css\" />";
        css += "<link href=\""+contextPath+"/plugin/org.joget.pbuilder.PageBuilder/flexiblegs-css.css?build="+getVersion()+buildNumber+"\" rel=\"stylesheet\" type=\"text/css\" />";
        return css;
    }

    @Override
    public String getBuilderHTML(BuilderDefinition def, String json, HttpServletRequest request, HttpServletResponse response) {
//        Map<String, Object> model = new HashMap<String, Object>();
//        model.put("definition", def);
//        model.put("json", json);
//        return CustomBuilderUtil.generateHtml(this, "page_builder.ftl", model, request);
        return "";
    }
    
    @Override
    public void builderPreview(String json, HttpServletRequest request, HttpServletResponse response) {
        Map<String, Object> config = new HashMap<String, Object>();
        config.put("preview", true);
        String html = (String) getBuilderResult(json, config);
        
        try {
            response.setContentType("text/html; charset=utf-8");
            
            Map<String, Object> model = new HashMap<String, Object>();
            model.put("html", html);
            html = CustomBuilderUtil.generateHtml(this, "preview.ftl", model, request);
            response.getWriter().write(html);
        } catch (IOException ex) {
            LogUtil.error(PageBuilder.class.getName(), ex, "");
        }
    }
    
    public static Collection<PageWidget> getAvailableElements () {
        PluginManager pluginManager = (PluginManager) AppUtil.getApplicationContext().getBean("pluginManager");
        Collection<PageWidget> list = new ArrayList<PageWidget>();
        Collection<Plugin> pluginList = pluginManager.list(PageWidget.class);
        for (Plugin plugin : pluginList) {
            if (plugin instanceof PageWidget) {
                list.add((PageWidget) plugin);
            }
        }
        return list;
    }

    @Override
    public void webService(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        boolean isAdmin = WorkflowUtil.isCurrentUserInRole(WorkflowUserManager.ROLE_ADMIN);
        if (!isAdmin) {
            response.sendError(HttpServletResponse.SC_UNAUTHORIZED);
            return;
        }
        
        String action = request.getParameter("_action");
        if ("previewElement".equals(action)) {
            String json = request.getParameter("_json");
            String id = request.getParameter("_id");
            AppDefinition appDef = AppUtil.getCurrentAppDefinition();

            String tempJson = json;
            if (tempJson.contains(SecurityUtil.ENVELOPE) || tempJson.contains(PropertyUtil.PASSWORD_PROTECTED_VALUE)) {
                BuilderDefinitionDao dao = (BuilderDefinitionDao) AppUtil.getApplicationContext().getBean("builderDefinitionDao");
                BuilderDefinition def = dao.loadById(id, appDef);

                if (def != null) {
                    tempJson = PropertyUtil.propertiesJsonStoreProcessing(def.getJson(), tempJson);
                }
            }
            
            String html = "";
            try {
                JSONObject obj = new JSONObject(tempJson);
                if (obj.has("className")) {
                    PluginManager pluginManager = (PluginManager) AppUtil.getApplicationContext().getBean("pluginManager");
                    Plugin plugin = pluginManager.getPlugin(obj.getString("className"));
                    if (plugin != null && plugin instanceof PageWidget && plugin instanceof PropertyEditable) {
                        ((PropertyEditable) plugin).setProperties(PropertyUtil.getPropertiesValueFromJson(obj.getJSONObject("properties").toString()));
                        html = ((PageWidget) plugin).render();
                    }
                }
            } catch (Exception ex) {
                LogUtil.error(this.getClass().getName(), ex, "preview element error!");
            }
            response.getWriter().write(html);
        } else {
            try {
                JSONArray jsonArray = new JSONArray();

                for (PageWidget p : getAvailableElements()) {
                    Map<String, String> option = new HashMap<String, String>();
                    option.put("className", p.getClassName());
                    option.put("icon", p.getIcon());
                    option.put("label", p.getLabel());
                    option.put("propertyOptions", p.getPropertyOptions());
                    option.put("defaultProperties", PropertyUtil.getDefaultPropertyValues(option.get("propertyOptions")));
                    jsonArray.put(option);
                }

                jsonArray.write(response.getWriter());
            } catch (Exception ex) {
                LogUtil.error(this.getClass().getName(), ex, "Get available page widgets plugin error!");
            }
        }
    }

    @Override
    public Object getBuilderResult(String json, Map<String, Object> config) {
        HttpServletRequest request = WorkflowUtil.getHttpServletRequest();
        boolean isPreview = false;
        if (config.containsKey("preview")) {
            isPreview = (boolean) config.get("preview");
        }
        
        String result = "<link href=\""+request.getContextPath()+"/plugin/org.joget.pbuilder.PageBuilder/flexiblegs-css.css\" rel=\"stylesheet\" type=\"text/css\" />";
        
        json = AppUtil.processHashVariable(json, null, StringUtil.TYPE_JSON, null);
        json = AppUtil.replaceAppMessages(json, StringUtil.TYPE_JSON);
        
        PluginManager pluginManager = (PluginManager) AppUtil.getApplicationContext().getBean("pluginManager");
        WorkflowUserManager workflowUserManager = (WorkflowUserManager) AppUtil.getApplicationContext().getBean("workflowUserManager");
        User currentUser = workflowUserManager.getCurrentUser();
        
        try {
            JSONObject obj = new JSONObject(json);
            String permissionKey = Permission.DEFAULT;
            boolean hasPermission = false;
            
            if (!isPreview) {
                if (obj.getJSONObject("properties").has("permission_rules")) {
                    JSONArray permissionRules = obj.getJSONObject("properties").getJSONArray("permission_rules");
                    if (permissionRules != null && permissionRules.length() > 0) {
                        for (int i = 0; i < permissionRules.length(); i++) {
                            JSONObject rule = permissionRules.getJSONObject(i);
                            if (rule.has("permission")) {
                                JSONObject permissionObj = rule.optJSONObject("permission");
                                hasPermission = CustomBuilderUtil.getPermisionResult(permissionObj.toString(), request.getParameterMap(), currentUser);
                                if (hasPermission) {
                                    permissionKey = rule.getString("permission_key");
                                    break;
                                }
                            }
                        }
                    }
                }
                
                if (!hasPermission) {
                    if (obj.getJSONObject("properties").has("permission")) {
                        JSONObject permissionObj = obj.getJSONObject("properties").getJSONObject("permission");
                        hasPermission = CustomBuilderUtil.getPermisionResult(permissionObj.toString(), request.getParameterMap(), currentUser);
                    } else {
                        hasPermission = true;
                    }
                }
            } else {
                hasPermission = true;
            }
            
            if (hasPermission) {
                result += renderRows(obj, permissionKey, request.getParameterMap(), currentUser, isPreview);
            } else {
                String noPermissionMessage = pluginManager.getMessage("PageBuilder.noPermission", getClassName(), MESSAGE_PATH);
                if (obj.getJSONObject("properties").has("noPermissionMessage") && !obj.getJSONObject("properties").getString("noPermissionMessage").isEmpty()) {
                    noPermissionMessage = obj.getJSONObject("properties").getString("noPermissionMessage");
                }
                result += "<h1>" + noPermissionMessage + "</h1>";
            }
        } catch (Exception e) {
            LogUtil.error(getClassName(), e, "");
        }
        
        return result;
    }
    
    private String renderRows(JSONObject element, String permissionKey, Map requestParams,User currentUser, boolean isPreview) throws JSONException {
        String result = "";
        
        JSONArray elementsArray = element.getJSONArray("elements");
        for (int i = 0; i < elementsArray.length(); i++) {
            JSONObject elementObj = (JSONObject) elementsArray.get(i);
            
            boolean hasPermission = false;
            if (!isPreview) {
                //check for permission
                JSONObject ruleObj = null;
                if (Permission.DEFAULT.equals(permissionKey)) {
                    ruleObj = elementObj.getJSONObject("properties");
                } else if (elementObj.getJSONObject("properties").has("permission_rules")) {
                    JSONObject permissionRules = elementObj.getJSONObject("properties").getJSONObject("permission_rules");
                    if (permissionRules != null && permissionRules.has(permissionKey)) {
                        ruleObj = permissionRules.getJSONObject(permissionKey);
                    }
                }
                
                if (ruleObj != null) {
                    if (ruleObj.has("permission")){
                        try {
                            JSONObject permissionObj = ruleObj.getJSONObject("permission");
                            hasPermission = CustomBuilderUtil.getPermisionResult(permissionObj.toString(), requestParams, currentUser);
                        } catch (Exception e) {
                            LogUtil.debug(getClass().getName(), "set category permission error.");
                        }
                    } else {
                        hasPermission = true;
                    }
                } else {
                    hasPermission = true;
                }
            } else {
                hasPermission = true;
            }
            
            if (hasPermission) {
                //build Row
                PageWidget row = getPageWidget(elementObj, permissionKey, requestParams, currentUser, isPreview);
                if (row != null) {
                    result += row.render();
                }
            }
        }
        return result;
    }
    
    private PageWidget getPageWidget(JSONObject element, String permissionKey, Map requestParams, User currentUser, boolean isPreview) {
        if (element.has("className")) {
            try {
                PluginManager pluginManager = (PluginManager) AppUtil.getApplicationContext().getBean("pluginManager");
                Plugin plugin = pluginManager.getPlugin(element.getString("className"));
                if (plugin != null && plugin instanceof PageWidget && plugin instanceof PropertyEditable) {
                    boolean visible = true;
                    
                    if (!isPreview) {
                        //check for permission rule
                        JSONObject ruleObj = null;
                        if (Permission.DEFAULT.equals(permissionKey)) {
                            ruleObj = element.getJSONObject("properties");
                        } else if (element.getJSONObject("properties").has("permission_rules")) {
                            JSONObject permissionRules = element.getJSONObject("properties").getJSONObject("permission_rules");
                            if (permissionRules != null && permissionRules.has(permissionKey)) {
                                ruleObj = permissionRules.getJSONObject(permissionKey);
                            }
                        }

                        if (ruleObj != null && ruleObj.has("hidden") && "true".equalsIgnoreCase(ruleObj.getString("hidden"))) {
                            visible = false;
                        }
                    }
                    
                    if (visible) {
                        ((PropertyEditable) plugin).setProperties(PropertyUtil.getPropertiesValueFromJson(element.getJSONObject("properties").toString()));
                    
                        //get childs
                        if (element.has("elements")) {
                            Collection<PageWidget> childs = new ArrayList<PageWidget>();
                            JSONArray childsArr = element.getJSONArray("elements");
                            for (int i = 0; i < childsArr.length(); i++) {
                                JSONObject childObj = (JSONObject) childsArr.get(i);
                                PageWidget c = getPageWidget(childObj, permissionKey, requestParams, currentUser, isPreview);
                                if (c != null) {
                                    childs.add(c);
                                }
                            }
                            ((PropertyEditable) plugin).setProperty("page_widgets", childs);
                        }
                        return (PageWidget) plugin;
                    }
                }
            } catch (Exception e) {
                LogUtil.error(getClassName(), e, "");
            }
        }
        return null;
    }
}
