package org.joget.pbuilder.lib;

import java.util.Collection;
import org.joget.apps.app.service.AppPluginUtil;
import org.joget.apps.app.service.AppUtil;
import org.joget.pbuilder.api.PageWidget;
import org.joget.plugin.base.ExtDefaultPlugin;
import org.joget.plugin.property.model.PropertyEditable;

public class Row extends ExtDefaultPlugin implements PageWidget, PropertyEditable {
    
    private final static String MESSAGE_PATH = "messages/PageBuilder";
    
    @Override
    public String getName() {
        return "RowWidget";
    }

    @Override
    public String getVersion() {
        return "7.0.0";
    }

    @Override
    public String getDescription() {
        return AppPluginUtil.getMessage("rowWidget.description", getClassName(), MESSAGE_PATH);
    }

    @Override
    public String getIcon() {
        return "";
    }

    @Override
    public String getLabel() {
        return AppPluginUtil.getMessage("rowWidget.label", getClassName(), MESSAGE_PATH);
    }

    @Override
    public String getClassName() {
        return getClass().getName();
    }

    @Override
    public String getPropertyOptions() {
        return AppUtil.readPluginResource(getClassName(), "/properties/page/rowWidget.json", null, true, MESSAGE_PATH);
    }
    
    @Override
    public String render() {
        String cssClass = "";
        if (!getPropertyString("desktopCols").isEmpty()) {
            cssClass += " xl-" + getPropertyString("desktopCols");
        }
        if (!getPropertyString("tabletCols").isEmpty()) {
            cssClass += " md-" + getPropertyString("tabletCols");
        }
        if (!getPropertyString("mobileCols").isEmpty()) {
            cssClass += " sm-" + getPropertyString("mobileCols");
        }
        
        String html = "<div class=\"wrap xl-gutter-24 "+cssClass+"\">";
        
        //load columns
        if (getProperty("page_widgets") != null) {
            Collection<PageWidget> childs = (Collection<PageWidget>) getProperty("page_widgets");
            if (!childs.isEmpty()) {
                for (PageWidget c : childs) {
                    ((PropertyEditable) c).setProperty("rowDesktopCols", getPropertyString("desktopCols"));
                    ((PropertyEditable) c).setProperty("rowTabletCols", getPropertyString("tabletCols"));
                    ((PropertyEditable) c).setProperty("rowMobileCols", getPropertyString("mobileCols"));
                    html += c.render();
                }
            }
        }
        
        html += "</div>";
        return html;
    }
}