PageBuilder = {
    currentRow : null,
    currentCol : null,
    
    initBuilder : function(callback) {
        
        PageBuilder.initPalette(function(){
            // make rows sortable
            $("#builder_canvas").sortable({
                handle: ".builder-row-handle",
                items: ".builder-row",
                stop: function(event, ui) {
                    var row = $(ui.item[0]);
                    var rowObj = $(row)[0].dom;
                    var oldIndex = $.inArray(rowObj, CustomBuilder.data.elements);
                    var newIndex = 0;
                    var prev = $(row).prev();
                    if ($(prev).length > 0) {
                        var prevObj = $(prev)[0].dom;
                        newIndex = $.inArray(prevObj, CustomBuilder.data.elements) + 1;
                    }
                    PageBuilder.arrayMove(CustomBuilder.data.elements, oldIndex, newIndex);

                    //update row sortable
                    CustomBuilder.update();
                },
                tolerance: "pointer",
                revertDuration: 100,
                revert: "invalid"
            }).disableSelection();

            $("#builder_canvas").off("click", ".element-add-row");
            $("#builder_canvas").on("click", ".element-add-row", function(){
                PageBuilder.addRow($(this).closest(".builder-row"));
            });

            if (callback !== null && callback !== undefined) {
                callback();
            }
        });
    },
    
    load : function(data) {
        $("#builder_canvas").html("");
        
        if (data.elements !== undefined && data.elements.length > 0) {
            for(var e in data.elements){
                PageBuilder.renderRow($("#builder_canvas"), data.elements[e], true);
            }
        } else {
            PageBuilder.renderRow($("#builder_canvas"));
        }
        
        CustomBuilder.update(false);
    },
    
    addRow : function(element, rowObj, append) {
        PageBuilder.renderRow(element, rowObj, append);
        CustomBuilder.update();
    },
    
    renderRow : function(element, rowObj, append) {
        if (rowObj === null || rowObj === undefined) {
            rowObj = {
                'className' : 'org.joget.pbuilder.lib.Row',
                'properties' : {
                    'id' : PageBuilder.uuid(),
                    'desktopCols' : "1",
                    'tabletCols' : "",
                    'mobileCols' : ""
                },
                'elements' : [
                    {
                        'className' : 'org.joget.pbuilder.lib.Column',
                        'properties' : {
                            "id" : PageBuilder.uuid(),
                            "desktopCols" : "",
                            "tabletCols" : "",
                            "MobileCols" : ""
                        },
                        'elements' : []
                    }
                ]
            };
            
            if (CustomBuilder.data.elements === undefined) {
                CustomBuilder.data.elements = [];
            }
            
            CustomBuilder.data.elements.push(rowObj);
        }
        
        var html = '<div class="builder-row"><div class="builder-row-handle"></div><div class="element-options corner white"></div><div class="wrap xl-gutter-24"></div><div class="element-options middle white"></div></div>';
        var row = $(html); 
        
        $(row).attr("id", rowObj["properties"]["id"]);
        $(row).find('> .element-options.corner').append('<button class="element-edit-row" title="'+get_cbuilder_msg("pageBuilder.editRow")+'"><i class="far fa-edit"></i><span>'+get_cbuilder_msg("pageBuilder.editRow")+'</span></button>');
        $(row).find('> .element-options.corner').append('<button class="element-add-col" title="'+get_cbuilder_msg("pageBuilder.addColumn")+'"><i class="fas fa-columns"></i><span>'+get_cbuilder_msg("pageBuilder.addColumn")+'</span></button>');
        $(row).find('> .element-options.corner').append('<button class="element-copy" title="'+get_cbuilder_msg('ubuilder.copy')+'"><i class="far fa-copy"></i><span>'+get_cbuilder_msg('ubuilder.copy')+'</span></button>');
        $(row).find('> .element-options.corner').append('<button class="element-delete" title="'+get_cbuilder_msg('pageBuilder.deleteRow')+'"><i class="fas fa-times"></i><span>'+get_cbuilder_msg('pageBuilder.deleteRow')+'</span></button>');
        
        $(row).find('> .element-options.middle').append('<button class="element-add-row" title="'+get_cbuilder_msg("pageBuilder.addRow")+'"><i class="fas fa-plus"></i><span>'+get_cbuilder_msg("pageBuilder.addRow")+'</span></button>');
        $(row).find('> .element-options.middle').append('<button class="element-paste row" title="'+get_cbuilder_msg("pageBuilder.pasteRow")+'"><i class="fas fa-paste"></i><span>'+get_cbuilder_msg("pageBuilder.pasteRow")+'</span></button>');
        
        $(row)[0].dom = rowObj;
        
        if (rowObj["properties"]["desktopCols"] !== "") {
            $(row).find('> .wrap').addClass("xl-" + rowObj["properties"]["desktopCols"]);
        }
        if (rowObj["properties"]["tabletCols"] !== "") {
            $(row).find('> .wrap').addClass("md-" + rowObj["properties"]["tabletCols"]);
        }
        if (rowObj["properties"]["mobileCols"] !== "") {
            $(row).find('> .wrap').addClass("sm-" + rowObj["properties"]["mobileCols"]);
        }
        
        PageBuilder.initRow(row);
        if (!$(element).hasClass("builder-row")) {
            if (append !== undefined && append) {
                $(element).append(row);
            } else {
                $(element).prepend(row);
            }
        } else {
            $(element).after(row);
        }
        
        //render columns
        if (rowObj !== undefined && rowObj['elements'] !== undefined) {
            $.each(rowObj['elements'], function(i, col) {
                PageBuilder.renderColumn(row, col);
            });
        }
    },
    
    initRow : function(row){
        $(row).find("> .element-options > .element-edit-row").on("click", function(){
            PageBuilder.showProperties(row);
        });
        
        $(row).find("> .element-options > .element-add-col").on("click", function(){
            PageBuilder.addColumn(row);
        });
        
        $(row).find("> .element-options > .element-delete").on("click", function(){
            PageBuilder.deleteRow(row);
        });
        
        $(row).find("> .element-options > .element-copy").on("click", function(){
            CustomBuilder.copy($(row)[0].dom, "row");
        });
        
        $(row).find("> .element-options > .element-paste.row").on("click", function(){
            PageBuilder.pasteRow(row);
        });
        
        $(row).find(".wrap").sortable({
            handle: ".builder-column-handle",
            opacity: 0.8,
            dropOnEmpty: true,
            activeClass: "col-highlight",
            connectWith: ".wrap",
            start: function( event, ui ) {
                var col = $(ui.item[0]);
                PageBuilder.currentRow = $(col).closest(".builder-row");
            },
            stop: function(event, ui){
                var col = $(ui.item[0]);
                var colObj = $(col)[0].dom;
                var row = $(col).closest(".builder-row");
                var rowObj = $(row)[0].dom;
                
                if (PageBuilder.currentRow === row) {
                    var oldIndex = $.inArray(colObj, rowObj.elements);
                    var newIndex = 0;
                    var prev = $(col).prev();
                    if ($(prev).length > 0) {
                        var prevObj = $(prev)[0].dom;
                        newIndex = $.inArray(prevObj, rowObj.elements) + 1;
                    }
                    PageBuilder.arrayMove(rowObj.elements, oldIndex, newIndex);
                } else {
                    var prevRowObj = $(PageBuilder.currentRow)[0].dom;
                    var index = $.inArray(colObj, prevRowObj.elements);
                    if (index !== -1) {
                        prevRowObj.elements.splice(index, 1);
                    }
                    var newIndex = 0;
                    var prev = $(row).prev();
                    if ($(prev).length > 0) {
                        var prevObj = $(prev)[0].dom;
                        newIndex = $.inArray(prevObj, rowObj.elements) + 1;
                    }
                    
                    rowObj.elements.splice(newIndex, 0, colObj);
                    
                    PageBuilder.updateRowCols(row);
                }
                
                //update row sortable
                CustomBuilder.update();
            }
        });
    },
    
    showProperties : function(element) {
        var dom = $(element)[0].dom;
        var elementClass = dom.className;
        var elementProperty = dom.properties;
        CustomBuilder.editProperties(elementClass, elementProperty);
    },
    
    saveEditProperties : function(container, properties) {
        var element = $("#"+properties["id"]);
        
        if ($(element).hasClass("builder-row")) {
            $(element).find('> .wrap').attr("class", "wrap xl-gutter-24 ui-sortable");
            if (dom.properties["desktopCols"] !== "") {
                $(element).find('> .wrap').addClass("xl-" + dom.properties["desktopCols"]);
            }
            if (dom.properties["tabletCols"] !== "") {
                $(element).find('> .wrap').addClass("md-" + dom.properties["tabletCols"]);
            }
            if (dom.properties["mobileCols"] !== "") {
                $(element).find('> .wrap').addClass("sm-" + dom.properties["mobileCols"]);
            }
        } else if ($(element).hasClass("builder-col")) {
            $(element).attr("class", "builder-col col");

            var row = $(element).closest(".builder-row");
            var rowObj = $(row)[0].dom;
            var cols = rowObj["properties"]["desktopCols"];

            if (dom.properties["desktopCols"] !== undefined && dom.properties["desktopCols"] !== "" && cols !== "") {
                $(element).addClass("xl-" + dom.properties["desktopCols"] + "-" + cols);
            }
            if (dom.properties["tabletCols"] !== undefined && dom.properties["tabletCols"] !== "") {
                if (rowObj["properties"]["tabletCols"] !== "") {
                    cols = rowObj["properties"]["tabletCols"];
                }
                $(element).addClass("md-" + dom.properties["tabletCols"] + "-" + cols);
            }
            if (dom.properties["mobileCols"] !== undefined && dom.properties["mobileCols"] !== "") {
                if (rowObj["properties"]["mobileCols"] !== "") {
                    cols = rowObj["properties"]["mobileCols"];
                }
                $(element).addClass("sm-" + dom.properties["mobileCols"] + "-" + cols);
            }
        } else {
            PageBuilder.reloadWidget($(element));
        }
    },
    
    deleteRow : function(row) {
        var rowObj = $(row)[0].dom;
        var index = $.inArray(rowObj, CustomBuilder.data.elements);
        if (index !== -1) {
            CustomBuilder.data.elements.splice(index, 1);
        }
        $(row).remove();
        
        CustomBuilder.update();
    },
    
    updatePasteId: function(element) {
        element["properties"]["id"] = PageBuilder.uuid();
        if (element["elements"] !== undefined && element["elements"].length > 0) {
            for (var i in element["elements"]) {
                PageBuilder.updatePasteId(element["elements"][i]);
            }
        }
    },
    
    pasteRow : function(row) {
        var copied = CustomBuilder.getCopiedElement();
        
        if (copied['type'] === "row") {
            var copiedRowObj = copied['object'];
            PageBuilder.updatePasteId(copiedRowObj);
            
            PageBuilder.addRow(row, copiedRowObj);
        }
    },
    
    addColumn : function(row, columnObj) {
        PageBuilder.renderColumn(row);
        PageBuilder.updateRowCols(row);
        CustomBuilder.update();
    },
    
    updateRowCols : function(row) {
        var rowObj = $(row)[0].dom;
        var ogiCols = 0;
        try {
            ogiCols = parseInt(rowObj["properties"]["desktopCols"]);
        } catch (err) {}
        var length = $(row).find("> .wrap > .builder-col").length;
        if (ogiCols < length) {
            if (length > 12) {
                length = 12;
            }
            $(row).find('> .wrap').removeClass("xl-" + rowObj["properties"]["desktopCols"]);
            rowObj["properties"]["desktopCols"] = ""+length;
            $(row).find('> .wrap').addClass("xl-" + rowObj["properties"]["desktopCols"]);
        }
    },
    
    renderColumn : function(row, columnObj) {
        var rowObj = $(row)[0].dom;
        if (columnObj === undefined) {
            columnObj = {
                'className' : 'org.joget.pbuilder.lib.Column',
                'properties' : {
                    "id" : PageBuilder.uuid(),
                    "desktopCols" : "",
                    "tabletCols" : "",
                    "MobileCols" : ""
                },
                'elements' : []
            };
            if (rowObj.elements === undefined) {
                rowObj.elements = [];
            }
            
            rowObj.elements.push(columnObj);
        }
        
        var html = '<div class="builder-col col"><div class="builder-col-inner"><div class="builder-column-handle"></div><div class="element-options corner white"></div><div class="widgets"></div></div></div>';
        var column = $(html); 
        
        $(column).attr("id", columnObj["properties"]["id"]);
        $(column).find('> .builder-col-inner > .element-options.corner').append('<button class="element-edit-col" title="'+get_cbuilder_msg("pageBuilder.editColumn")+'"><i class="far fa-edit"></i><span>'+get_cbuilder_msg("pageBuilder.editColumn")+'</span></button>');
        $(column).find('> .builder-col-inner > .element-options.corner').append('<button class="element-paste widget" title="'+get_cbuilder_msg("pageBuilder.pasteWidget")+'"><i class="fas fa-paste"></i><span>'+get_cbuilder_msg("pageBuilder.pasteWidget")+'</span></button>');
        $(column).find('> .builder-col-inner > .element-options.corner').append('<button class="element-delete" title="'+get_cbuilder_msg('pageBuilder.deleteColumn')+'"><i class="fas fa-times"></i><span>'+get_cbuilder_msg('pageBuilder.deleteColumn')+'</span></button>');
        
        
        var cols = rowObj["properties"]["desktopCols"];
        
        if (columnObj["properties"]["desktopCols"] !== undefined && columnObj["properties"]["desktopCols"] !== "" && cols !== "") {
            $(column).addClass("xl-" + columnObj["properties"]["desktopCols"] + "-" + cols);
        }
        if (columnObj["properties"]["tabletCols"] !== undefined && columnObj["properties"]["tabletCols"] !== "") {
            if (rowObj["properties"]["tabletCols"] !== "") {
                cols = rowObj["properties"]["tabletCols"];
            }
            $(column).addClass("md-" + columnObj["properties"]["tabletCols"] + "-" + cols);
        }
        if (columnObj["properties"]["mobileCols"] !== undefined && columnObj["properties"]["mobileCols"] !== "") {
            if (rowObj["properties"]["mobileCols"] !== "") {
                cols = rowObj["properties"]["mobileCols"];
            }
            $(column).addClass("sm-" + columnObj["properties"]["mobileCols"] + "-" + cols);
        }
        
        $(column)[0].dom = columnObj;
        
        $(column).find('> .builder-col-inner > .element-options .element-edit-col').on("click", function(){
            PageBuilder.showProperties(column);
        });
        
        $(column).find('> .builder-col-inner > .element-options .element-paste').on("click", function(){
            PageBuilder.pasteWidget(column);
        });
        
        $(column).find('> .builder-col-inner > .element-options .element-delete').on("click", function(){
            PageBuilder.deleteColumn(column);
        });
        
        $(row).find('> .wrap').append(column);
        
        $(column).find(".widgets").sortable({
            opacity: 0.8,
            dropOnEmpty: true,
            activeClass: "widget-highlight",
            connectWith: ".widgets",
            start: function( event, ui ) {
                var widget = $(ui.item[0]);
                if (!$(widget).hasClass("builder-palette-element")) {
                    PageBuilder.currentCol = $(widget).closest(".builder-col");
                } else {
                    PageBuilder.currentCol = null;
                }
            },
            stop: function(event, ui){
                var widget = $(ui.item[0]);
                var col = $(widget).closest(".builder-col");
                var colObj = $(col)[0].dom;
                
                if ($(widget).hasClass("builder-palette-element")) {
                    PageBuilder.addWidget(col, widget);
                } else {
                    var widgetObj = $(widget)[0].dom;
                    if (PageBuilder.currentCol[0] === col[0]) {
                        var oldIndex = $.inArray(widgetObj, colObj.elements);
                        var newIndex = 0;
                        var prev = $(widget).prev();
                        if ($(prev).length > 0) {
                            var prevObj = $(prev)[0].dom;
                            newIndex = $.inArray(prevObj, colObj.elements) + 1;
                        }
                        PageBuilder.arrayMove(colObj.elements, oldIndex, newIndex);
                    } else {
                        var prevColObj = $(PageBuilder.currentCol)[0].dom;
                        var index = $.inArray(widgetObj, prevColObj.elements);
                        if (index !== -1) {
                            prevColObj.elements.splice(index, 1);
                        }
                        var newIndex = 0;
                        var prev = $(widget).prev();
                        if ($(prev).length > 0) {
                            var prevObj = $(prev)[0].dom;
                            newIndex = $.inArray(prevObj, prevColObj.elements) + 1;
                        }

                        colObj.elements.splice(newIndex, 0, widgetObj);
                    }
                }
                
                //update row sortable
                CustomBuilder.update();
            }
        });
        
        //render widgets
        if (columnObj !== undefined && columnObj['elements'] !== undefined) {
            $.each(columnObj['elements'], function(i, widget) {
                PageBuilder.renderWidget(column, widget);
            });
        }
    },
    
    addWidget : function(column, tempElement) {
        var classname = $(tempElement).attr("element-class");
        var properties = CustomBuilder.paletteElements[classname].properties;
        if (properties === undefined) {
            properties = {};
        }
        properties["id"] = PageBuilder.uuid();
        PageBuilder.renderWidget(column, {
            className : classname,
            properties : properties
        } ,tempElement);
        CustomBuilder.update();
    },
    
    pasteWidget : function(column) {
        var copied = CustomBuilder.getCopiedElement();
        if (copied['type'] === "widget") {
            var copiedObj = copied['object'];
            PageBuilder.updatePasteId(copiedObj);
            var temp = $('<div></div>');
            $(column).find(".widgets").append(temp);
            PageBuilder.renderWidget(column, copiedObj, temp);
            CustomBuilder.update();
        }
    },
    
    renderWidget : function(column, widgetObj, tempElement){
        var columnObj = $(column)[0].dom;
        if (widgetObj === undefined) {
            return;
        }
        if (tempElement !== undefined) {
            var index = $(column).find(".widgets > div").index($(tempElement));
            columnObj.elements.splice(index, 0, widgetObj);
        }
        
        var html = '<div class="builder-widget"><div class="builder-widget-handle"><div class="plugin-label"></div></div><div class="element-options middle"></div><div class="widget-template"></div></div>';
        var widget = $(html); 
        $(widget)[0].dom = widgetObj;
        
        $(widget).find('.plugin-label').text(CustomBuilder.paletteElements[widgetObj['className']].label);
        
        $(widget).attr("id", widgetObj["properties"]["id"]);
        $(widget).find('> .element-options').append('<button class="element-edit-widget" title="'+get_cbuilder_msg("pageBuilder.editWidget")+'"><i class="far fa-edit"></i><span>'+get_cbuilder_msg("pageBuilder.editWidget")+'</span></button>');
        $(widget).find('> .element-options').append('<button class="element-copy widget" title="'+get_cbuilder_msg("pageBuilder.copyWidget")+'"><i class="far fa-copy"></i><span>'+get_cbuilder_msg("pageBuilder.copyWidget")+'</span></button>');
        $(widget).find('> .element-options').append('<button class="element-delete" title="'+get_cbuilder_msg('pageBuilder.deleteWidget')+'"><i class="fas fa-times"></i><span>'+get_cbuilder_msg('pageBuilder.deleteWidget')+'</span></button>');
        
        $(widget).find('> .element-options .element-edit-widget').on("click", function(){
            PageBuilder.showProperties(widget);
        });
        
        $(widget).find('> .element-options .element-copy').on("click", function(){
            CustomBuilder.copy(widgetObj, "widget");
        });
        
        $(widget).find('> .element-options .element-delete').on("click", function(){
            PageBuilder.deleteWidget(widget);
        });
        
        PageBuilder.reloadWidget(widget);
        
        if (tempElement !== undefined) {
            $(tempElement).replaceWith(widget);
        } else {
            $(column).find(".widgets").append(widget);
        }
    },
    
    reloadWidget : function(widget) {
        var widgetObj = $(widget)[0].dom;
        $(widget).find('.widget-template').html('<i class="fas fa-spinner fa-spin"></i>');
        
        $.post({
            url: CustomBuilder.contextPath + '/web/json/app/'+CustomBuilder.appId+'/'+CustomBuilder.appVersion+'/plugin/org.joget.pbuilder.PageBuilder/service',
            data : {
                _id : CustomBuilder.data.properties.id,
                _json : JSON.encode(widgetObj),
                _action : 'previewElement'
            },
            dataType : "text",
            success: function(response) {
                if (response !== null) {
                    $(widget).find('.widget-template').html(response);
                }
            }
        });
    },
    
    deleteWidget : function(widget) {
        var col = $(widget).closest(".builder-col");
        var colObj = $(col)[0].dom;
        
        var widgetObj = $(widget)[0].dom;
        var index = $.inArray(widgetObj, colObj.elements);
        if (index !== -1) {
            colObj.elements.splice(index, 1);
        }
        $(widget).remove();
        
        CustomBuilder.update();
    },
    
    deleteColumn : function(column) {
        var row = $(column).closest(".builder-row");
        var rowObj = $(row)[0].dom;
        
        var columnObj = $(column)[0].dom;
        var index = $.inArray(columnObj, rowObj.elements);
        if (index !== -1) {
            rowObj.elements.splice(index, 1);
        }
        $(column).remove();
        
        if (rowObj.elements.length === 0) {
            CustomBuilder.renderColumn(row);
        }
        
        CustomBuilder.update();
    },
    
    initPalette : function(callback) {
        $.ajax({
            url: CustomBuilder.contextPath + '/web/json/plugin/org.joget.pbuilder.PageBuilder/service',
            dataType : "json",
            success: function(response) {
                if (response !== null && response !== "") {
                    for (var i in response) {
                        var p = response[i];
                        if (p.className !== "") {
                            var render = true;
                            if (p.className === "org.joget.pbuilder.lib.Column" || p.className === "org.joget.pbuilder.lib.Row") {
                                render = false;
                            }
                            CustomBuilder.initPaletteElement("", p.className, p.label, p.icon, p.propertyOptions, p.defaultProperties, render);
                        }
                    }
                    
                    //elements drag
                    $('.builder-palette-element').draggable({
                        connectToSortable: ".widgets",
                        opacity: 0.7,
                        helper: "clone",
                        zIndex: 200,
                        revert: "invalid",
                        cursor: "move"
                    }).disableSelection();
                }
                
                if (callback !== null && callback !== undefined) {
                    callback();
                }
            }
        });
    },
    
    arrayMove : function(arr, old_index, new_index) {
        if (new_index >= arr.length) {
            var k = new_index - arr.length + 1;
            while (k--) {
                arr.push(undefined);
            }
        }
        arr.splice(new_index, 0, arr.splice(old_index, 1)[0]);
        return arr; 
    },
    
    uuid : function(){
        return 'xxxxxxxx-xxxx-4xxx-xxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function(c) {  //xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx
            var r = Math.random()*16|0, v = c == 'x' ? r : (r&0x3|0x8);
            return v.toString(16);
        }).toUpperCase();
    },
};
